#include <stdio.h>
#include <stdlib.h>

char petscii2ascii(unsigned char c)
{
	if (c=='/' || c=='<' || c=='>' || c==' ' || c=='.' || c==0xA0 || c=='\''
		|| c=='-' || c=='+')
		return '_';
	/*if ((c >= 'A') && (c <= 'Z') || (c >= 'a') && (c <= 'z'))
		return c ^ 0x20;*/
	if ((c >= 0xc1) && (c <= 0xda))
		return c ^ 0x80;
	return c;
}

char *petstring2ascstring(unsigned char *str, unsigned int len)
{
	unsigned char *p = str;

	while (*p && len--) {
		*p = petscii2ascii(*p);
		p++;
	}

	return str;
}

void print_usage()
{
	printf("TSdump v1.0 by Gaia/LOD Jun 26, 2006\n");
	printf("This small program dumps track/sector data from a D64 file\n");
	printf("to be included into the source of a trackloader in assembler.\n");
	printf("Syntax: tsdump <d64_image>\n");
}

int main(int argc, char **argv)
{
	unsigned char buffer[256];
	FILE *d64;
	int  result;
	const unsigned int bam_offset = 0x16600;

	if (argc != 2 ) {
		print_usage();
		exit(1);
	}

	d64 = fopen(argv[1],"rb");
	if (!d64) {
		perror("ERROR. Could not open disk image file.");
		exit(2);
	}
	result = fseek(d64, bam_offset, SEEK_SET);
	if(result) {
		fclose(d64);
		perror("ERROR. Seek to BAM failed.");
		exit(3);
	}

	do {
		unsigned int entries = 0;
		unsigned char *ptr = buffer;

		fread(buffer, 256, 1, d64);
		if (buffer[0]) {
			result = fseek(d64, bam_offset + (buffer[1] - 1) * 256, SEEK_SET);
			fprintf(stderr, "Seeking track: %i, sector: %i.\n", buffer[0], buffer[1]);
			if(result) {
				fclose(d64);
				perror("ERROR. Seek to next BAM sector failed.");
				exit(4);
			}
		}
		ptr = buffer;
		while (ptr[2] == 0x82 && entries++<8) {
			char *filename = petstring2ascstring(ptr + 5, 16);
			printf("%s_TRACK  = %02i\n", filename,	ptr[3]);
			printf("%s_SECTOR = %02i\n", filename, ptr[4]);
			ptr += 32;
		}

	} while (buffer[0]);

	fclose(d64);
	return 0;
}
